/**
 * Test case for create.
 * Runs with mocha.
 */
"use strict";

const assert = require('assert'),
    create = require('../lib/create.js'),
    freeport = require('freeport'),
    async = require('async'),
    fs = require('fs'),
    apemandb = require('apemandb'),
    apemanApp = require('apeman-app'),
    request = require('request');

let tmpDir = __dirname + '/../tmp',
    dbFile = tmpDir + '/testing-db01.db';

describe('create', () => {
    let db;
    let baseUrl;
    let close;
    let req = request.defaults({jar: true});

    before((done) => {
        db = apemandb({
            env: {
                DATABASE: 'testing-database02',
                USERNAME: 'FOO',
                PASSWORD: 'BAR',
                DIALECT: 'sqlite',
                STORAGE: dbFile,
                LOGGING: false
            },
            models: __dirname + '/../doc/mocks/models/*.json'
        });
        db.sync({force: true})
            .then((db) => {
                done();
            });
    });

    before((done) => {
        let app = create(
            db.model('User'),
            db.model('Sign'),
            {
                passwordSpec: 'pdkdf2:120:184'
            });
        assert.ok(app);
        freeport((err, port) => {
            assert.ifError(err);
            apemanApp('testing-app', {
                configuration: {
                    $apps: {
                        'testing-app': {
                            '/': [
                                require('apeman-app-json')(),
                                require('apeman-app-format')(),
                                require('apeman-app-send')(),
                                require('apeman-app-header')(),
                                require('apeman-app-form')(),
                                require('apeman-app-session')(
                                    db.model('Session'), '1234qwer'
                                ),
                                require('apeman-app-signed')()
                            ],
                            '/api/sign': app
                        }
                    }
                },
                port: port
            }, (err, server) => {
                assert.ifError(err);
                close = server.close.bind(server);
                baseUrl = 'http://localhost:' + port + '/';
                done(err);
            });
        });
    });

    after((done) => {
        close(()=> {

            setTimeout(() => {
                fs.unlink(dbFile, () => {
                    done();
                });
            }, 100); //Wait for flush.
        });
    });

    it('signgetFail', (done) => {
        req({
            url: baseUrl + 'api/sign/signget',
            method: 'GET'
        }, (err, res, body) => {
            assert.equal(res.statusCode, 404);
            done(err);
        })
    });

    it('signupFail', (done) => {
        req({
            url: baseUrl + 'api/sign/signup',
            method: 'POST',
            form: {
                username: 'foo',
                password: 'foo_pswd'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 400);
            done(err);
        })
    });

    it('signupSuccess', (done) => {
        req({
            url: baseUrl + 'api/sign/signup',
            method: 'POST',
            form: {
                username: 'foo',
                email: 'foo@exampole.com',
                password: 'foo_pswd'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            done(err);
        })
    });

    it('signinSuccess', (done) => {
        req({
            url: baseUrl + 'api/sign/signin',
            method: 'POST',
            form: {
                sign_key: 'foo',
                password: 'foo_pswd'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let data = JSON.parse(body);
            assert.equal(data.success, true);
            done(err);
        })
    });

    it('signgetSuccess', (done) => {
        req({
            url: baseUrl + 'api/sign/signget',
            method: 'GET'
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let signed = JSON.parse(body);
            assert.ok(signed);
            done(err);
        })
    });

    it('signmodSucess', (done) => {
        req({
            url: baseUrl + 'api/sign/signmod',
            method: 'POST',
            form: {
                password: 'baz'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            done(err);
        })
    });

    it('signoutSuccess', (done) => {
        req({
            url: baseUrl + 'api/sign/signout',
            method: 'POST'
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let data = JSON.parse(body);
            assert.ok(data.signed.signout_at);
            done(err);
        })
    });

    it('signinFail', (done) => {
        req({
            url: baseUrl + 'api/sign/signin',
            method: 'POST',
            form: {}
        }, (err, res, body) => {
            assert.equal(res.statusCode, 400);
            let data = JSON.parse(body);
            let errors = data.errors;
            assert.equal(errors.sign_key[0].reason, 'OBJECT_REQUIRED');
            done(err);
        })
    });

    it('signinSuccess', (done) => {
        req({
            url: baseUrl + 'api/sign/signin',
            method: 'POST',
            form: {
                sign_key: 'foo',
                password: 'baz'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let data = JSON.parse(body);
            assert.equal(data.success, true);
            done(err);
        })
    });

    it('signdelSuccess', (done) => {
        req({
            url: baseUrl + 'api/sign/signdel',
            method: 'POST'
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            done(err);
        })
    });
});

